@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: fit.c,v 1.145.2.6 2014/11/08 04:52:25 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: fit.c,v 1.145.2.12 2015/07/09 01:40:44 sfeam Exp $"); }
 #endif
 
 /*  NOTICE: Change of Copyright Status
@@ -185,6 +185,7 @@ typedef enum marq_res marq_res_t;
 
 /* fit control */
 char *fitlogfile = NULL;
+TBOOLEAN fit_suppress_log = FALSE;
 TBOOLEAN fit_errorvariables = TRUE;
 TBOOLEAN fit_covarvariables = FALSE;
 verbosity_level fit_verbosity = BRIEF;
@@ -841,10 +842,12 @@ regress_finalize(int iter, double chisq, double last_chisq, double lambda, doubl
 	fit_show_brief(-2, chisq, chisq, a, lambda, STANDARD);
 
     /* tsm patchset 230: final progress report to log file */
-    if (fit_verbosity == VERBOSE)
-	fit_show(iter, chisq, last_chisq, a, lambda, log_f);
-    else
-	fit_show_brief(iter, chisq, last_chisq, a, lambda, log_f);
+    if (!fit_suppress_log) {
+	if (fit_verbosity == VERBOSE)
+	    fit_show(iter, chisq, last_chisq, a, lambda, log_f);
+	else
+	    fit_show_brief(iter, chisq, last_chisq, a, lambda, log_f);
+    }
 
     /* test covariance matrix */
     if (covar != NULL) {
@@ -971,9 +974,7 @@ regress_finalize(int iter, double chisq, double last_chisq, double lambda, doubl
 	}
     }
 
-    /* Report final results for VERBOSE, BRIEF, and RESULTS verbosity levels. */
-    if (fit_verbosity != QUIET)
-	show_results(chisq, last_chisq, a, dpar, corel);
+    show_results(chisq, last_chisq, a, dpar, corel);
 
     free(dpar);
     free_matr(corel);
@@ -1068,7 +1069,8 @@ regress(double a[])
 
     fit_show_lambda = TRUE;
     fit_progress(iter, chisq, chisq, a, lambda, STANDARD);
-    fit_progress(iter, chisq, chisq, a, lambda, log_f);
+    if (!fit_suppress_log)
+	fit_progress(iter, chisq, chisq, a, lambda, log_f);
 
     regress_init();
 
@@ -1413,7 +1415,7 @@ static void
 setvarcovar(char *varname1, char *varname2, double value)
 {
     /* The name of the (new) covariance variable */
-    char * pCovValName = (char *) gp_alloc(strlen(varname1) + strlen(varname2) + 6, "setvarcovar");
+    char * pCovValName = (char *) gp_alloc(strlen(varname1) + strlen(varname2) + 10, "setvarcovar");
     sprintf(pCovValName, "FIT_COV_%s_%s", varname1, varname2);
     setvar(pCovValName, value);
     free(pCovValName);
@@ -1864,24 +1866,23 @@ fit_command()
 	range_max[i] = -VERYLARGE;
 	range_autoscale[i] = AUTOSCALE_BOTH;
     }
-    range_min[MAX_NUM_VAR] = axis_array[z_axis].min;
-    range_max[MAX_NUM_VAR] = axis_array[z_axis].max;
-    range_autoscale[MAX_NUM_VAR] = axis_array[z_axis].autoscale;
+    range_min[iz] = axis_array[z_axis].min;
+    range_max[iz] = axis_array[z_axis].max;
+    range_autoscale[iz] = axis_array[z_axis].autoscale;
 
-    i = 0;
+    num_ranges = 0;
     while (equals(c_token, "[")) {
 	if (i > MAX_NUM_VAR)
 	    Eexc(c_token, "too many range specifiers");
 	/* NB: This has nothing really to do with the Z axis, we're */
 	/* just using that slot of the axis array as scratch space. */
 	AXIS_INIT3D(SECOND_Z_AXIS, 0, 1);
-	dummy_token[i] = parse_range(SECOND_Z_AXIS);
-	range_min[i] = axis_array[SECOND_Z_AXIS].min;
-	range_max[i] = axis_array[SECOND_Z_AXIS].max;
-	range_autoscale[i] = axis_array[SECOND_Z_AXIS].autoscale;
-	i++;
+	dummy_token[num_ranges] = parse_range(SECOND_Z_AXIS);
+	range_min[num_ranges] = axis_array[SECOND_Z_AXIS].min;
+	range_max[num_ranges] = axis_array[SECOND_Z_AXIS].max;
+	range_autoscale[num_ranges] = axis_array[SECOND_Z_AXIS].autoscale;
+	num_ranges++;
     }
-    num_ranges = i;
 
     /* now compile the function */
     token1 = c_token;
@@ -2060,17 +2061,22 @@ fit_command()
      * spec may be for the Z axis */
     if (num_ranges > num_indep+1)
 	Eexc2(dummy_token[num_ranges-1], "Too many range-specs for a %d-variable fit", num_indep);
+    if (num_ranges == (num_indep + 1)) {
+	/* last range was actually for the dependent variable */
+	range_min[iz] = range_min[num_indep];
+	range_max[iz] = range_max[num_indep];
+	range_autoscale[iz] = range_autoscale[num_indep];
+    }
 
     /* defer actually reading the data until we have parsed the rest
      * of the line */
     token3 = c_token;
 
     /* open logfile before we use any Dblfn calls */
-    {
+    if (!fit_suppress_log) {
 	char *logfile = getfitlogfile();
 	if ((logfile != NULL) && !log_f && !(log_f = fopen(logfile, "a")))
 	    Eex2("could not open log-file %s", logfile);
-	FPRINTF((STANDARD, "log-file=%s\n", logfile));
 	free(logfile);
     }
 
@@ -2113,12 +2119,13 @@ fit_command()
     FPRINTF((STANDARD, "prescale=%i\n", fit_prescale));
     FPRINTF((STANDARD, "errorscaling=%i\n", fit_errorscaling));
 
-    fputs("\n\n*******************************************************************************\n", log_f);
     (void) time(&timer);
-    fprintf(log_f, "%s\n\n", ctime(&timer));
-    {
+    if (!fit_suppress_log) {
 	char *line = NULL;
 
+	fputs("\n\n*******************************************************************************\n", log_f);
+	fprintf(log_f, "%s\n\n", ctime(&timer));
+
 	m_capture(&line, token2, token3 - 1);
 	fprintf(log_f, "FIT:    data read from %s\n", line);
 	fprintf(log_f, "        format = ");
@@ -2133,12 +2140,13 @@ fit_command()
     }
 
     /* report all range specs, starting with Z */
-    i = iz;
-    if ((range_autoscale[i] & AUTOSCALE_BOTH) != AUTOSCALE_BOTH)
-	    log_axis_restriction(log_f, i, range_min[i], range_max[i], range_autoscale[i], "function");
-    for (i = 0; i < num_indep; i++) {
-	if ((range_autoscale[i] & AUTOSCALE_BOTH) != AUTOSCALE_BOTH)
-	    log_axis_restriction(log_f, i, range_min[i], range_max[i], range_autoscale[i], c_dummy_var[i]);
+    if (!fit_suppress_log) {
+	if ((range_autoscale[iz] & AUTOSCALE_BOTH) != AUTOSCALE_BOTH)
+	    log_axis_restriction(log_f, iz, range_min[iz], range_max[iz], range_autoscale[iz], "function");
+	for (i = 0; i < num_indep; i++) {
+	    if ((range_autoscale[i] & AUTOSCALE_BOTH) != AUTOSCALE_BOTH)
+		log_axis_restriction(log_f, i, range_min[i], range_max[i], range_autoscale[i], c_dummy_var[i]);
+	}
     }
 
     /* start by allocting memory for MAX_DATA datapoints */
@@ -2322,14 +2330,12 @@ fit_command()
     redim_vec(&fit_z, num_data);
     redim_vec(&err_data, num_data * GPMAX(num_errors, 1));
 
-    fprintf(log_f, "        #datapoints = %d\n", num_data);
 
-    if (num_errors == 0)
-	fputs("        residuals are weighted equally (unit weight)\n\n", log_f);
-
-    {
+    if (!fit_suppress_log) {
 	char *line = NULL;
-
+	fprintf(log_f, "        #datapoints = %d\n", num_data);
+	if (num_errors == 0)
+	    fputs("        residuals are weighted equally (unit weight)\n\n", log_f);
 	m_capture(&line, token1, token2 - 1);
 	fprintf(log_f, "function used for fitting: %s\n", line);
 	print_function_definitions(func.at, log_f);
@@ -2357,10 +2363,11 @@ fit_command()
 
 	static char *viafile = NULL;
 	free(viafile);			/* Free previous name, if any */
-	viafile = try_to_get_string();	/* Cannot fail since isstringvalue succeeded */
-	fprintf(log_f, "fitted parameters and initial values from file: %s\n\n", viafile);
-	if (!(f = loadpath_fopen(viafile, "r")))
+	viafile = try_to_get_string();
+	if (!viafile || !(f = loadpath_fopen(viafile, "r")))
 	    Eex2("could not read parameter-file \"%s\"", viafile);
+	if (!fit_suppress_log)
+	    fprintf(log_f, "fitted parameters and initial values from file: %s\n\n", viafile);
 
 	/* get parameters and values out of file and ignore fixed ones */
 
@@ -2369,7 +2376,8 @@ fit_command()
 		break;
 	    if ((tmp = strstr(s, GP_FIXED)) != NULL) {	/* ignore fixed params */
 		*tmp = NUL;
-		fprintf(log_f, "FIXED:  %s\n", s);
+		if (!fit_suppress_log)
+		    fprintf(log_f, "FIXED:  %s\n", s);
 		fixed = TRUE;
 	    } else
 		fixed = FALSE;
@@ -2425,7 +2433,8 @@ fit_command()
     } else {
 	/* not a string after via: it's a variable listing */
 
-	fputs("fitted parameters initialized with current variable values\n\n", log_f);
+	if (!fit_suppress_log)
+	    fputs("fitted parameters initialized with current variable values\n\n", log_f);
 	do {
 	    if (!isletter(c_token))
 		Eex("no parameter specified");
@@ -2484,7 +2493,8 @@ fit_command()
     else
 	regress(a);	/* fit */
 
-    fclose(log_f);
+    if (log_f)
+	fclose(log_f);
     log_f = NULL;
     free(fit_x);
     free(fit_z);
@@ -2536,18 +2546,23 @@ Dblfn(const char *fmt, va_dcl)
     if (fit_verbosity != QUIET)
 	vfprintf(STANDARD, fmt, args);
     va_end(args);
-    VA_START(args, fmt);
-    vfprintf(log_f, fmt, args);
+    if (!fit_suppress_log) {
+	VA_START(args, fmt);
+	vfprintf(log_f, fmt, args);
+    }
 # else
     if (fit_verbosity != QUIET)
 	_doprnt(fmt, args, STANDARD);
-    _doprnt(fmt, args, log_f);
+    if (!fit_suppress_log) {
+	_doprnt(fmt, args, log_f);
+    }
 # endif
     va_end(args);
 #else
     if (fit_verbosity != QUIET)
 	fprintf(STANDARD, fmt, a1, a2, a3, a4, a5, a6, a7, a8);
-    fprintf(log_f, fmt, a1, a2, a3, a4, a5, a6, a7, a8);
+    if (!fit_suppress_log)
+	fprintf(log_f, fmt, a1, a2, a3, a4, a5, a6, a7, a8);
 #endif /* VA_START */
 }
 
