@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: parse.c,v 1.66.2.7 2014/07/12 23:58:43 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: parse.c,v 1.88.2.5 2015/08/05 15:50:28 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - parse.c */
@@ -38,17 +38,22 @@ static char *RCSid() { return RCSid("$Id: parse.c,v 1.66.2.7 2014/07/12 23:58:43
 
 #include "alloc.h"
 #include "command.h"
+#include "datablock.h"
 #include "eval.h"
 #include "help.h"
 #include "util.h"
 
-/* protection mechanism for parsing string followed by + or - sign */
+/* Protection mechanism for trying to parse a string followed by a + or - sign.
+ * Also suppresses an undefined variable message if an unrecognized token
+ * is encountered during try_to_get_string().
+ */
+TBOOLEAN string_result_only = FALSE;
 static int parse_recursion_level;
-static TBOOLEAN string_result_only = FALSE;
 
 /* Exported globals: the current 'dummy' variable names */
 char c_dummy_var[MAX_NUM_VAR][MAX_ID_LEN+1];
 char set_dummy_var[MAX_NUM_VAR][MAX_ID_LEN+1] = { "x", "y" };
+int  fit_dummy_var[MAX_NUM_VAR];
 TBOOLEAN scanning_range_in_progress = FALSE;
 
 /* This is used by plot_option_using() */
@@ -75,6 +80,7 @@ static void accept_exclusive_OR_expression __PROTO((void));
 static void accept_AND_expression __PROTO((void));
 static void accept_equality_expression __PROTO((void));
 static void accept_relational_expression __PROTO((void));
+static void accept_bitshift_expression __PROTO((void));
 static void accept_additive_expression __PROTO((void));
 static void accept_multiplicative_expression __PROTO((void));
 static void parse_primary_expression __PROTO((void));
@@ -86,6 +92,7 @@ static void parse_exclusive_OR_expression __PROTO((void));
 static void parse_AND_expression __PROTO((void));
 static void parse_equality_expression __PROTO((void));
 static void parse_relational_expression __PROTO((void));
+static void parse_bitshift_expression __PROTO((void));
 static void parse_additive_expression __PROTO((void));
 static void parse_multiplicative_expression __PROTO((void));
 static void parse_unary_expression __PROTO((void));
@@ -128,6 +135,7 @@ void
 parse_reset_after_error()
 {
     string_result_only = FALSE;
+    parse_recursion_level = 0;
 }
 
 /* JW 20051126:
@@ -163,9 +171,9 @@ const_express(struct value *valptr)
     return (valptr);
 }
 
-/* Used by plot2d/plot3d/fit:
- * Parse an expression that may return a string or may return a constant or may
- * be a dummy function using dummy variables x, y, ...
+/* Used by plot2d/plot3d/stats/fit:
+ * Parse an expression that may return a filename string, a datablock name,
+ * a constant, or a dummy function using dummy variables x, y, ...
  * If any dummy variables are present, set (*atptr) to point to an action table
  * corresponding to the parsed expression, and return NULL.
  * Otherwise evaluate the expression and return a string if there is one.
@@ -188,10 +196,12 @@ string_or_express(struct at_type **atptr)
     if (END_OF_COMMAND)
 	int_error(c_token, "expression expected");
 
-    if (isstring(c_token)) {
-	str = try_to_get_string();
+    /* parsing for datablocks */
+    if (equals(c_token,"$"))
+	return parse_datablock_name();
+
+    if (isstring(c_token) && (str = try_to_get_string()))
 	return str;
-    }
 
     /* parse expression */
     temp_at();
@@ -212,8 +222,15 @@ string_or_express(struct at_type **atptr)
 	struct value val;
 
 	evaluate_at(at, &val);
-	if (!undefined && val.type == STRING)
-	    str = val.v.string_val;
+	if (!undefined && val.type == STRING) {
+	    /* prevent empty string variable from treated as special file '' or "" */
+	    if (*val.v.string_val == '\0') {
+		free(val.v.string_val);
+		str = strdup(" ");
+	    } else {
+		str = val.v.string_val;
+	    }
+	}
     }
 
     /* prepare return */
@@ -369,11 +386,19 @@ accept_equality_expression()
 static void
 accept_relational_expression()
 {
-    accept_additive_expression();
+    accept_bitshift_expression();
     parse_relational_expression();
 }
 
 
+static void
+accept_bitshift_expression()
+{
+    accept_additive_expression();
+    parse_bitshift_expression();
+}
+
+
 static void
 accept_additive_expression()
 {
@@ -435,13 +460,20 @@ parse_primary_expression()
     } else if (equals(c_token, "$")) {
 	struct value a;
 
-	if (!isanumber(++c_token))
+	c_token++;
+	if (equals(c_token,"N")) {	/* $N == pseudocolumn -3 means "last column" */
+	    c_token++;
+	    Ginteger(&a, -3);
+	    at_highest_column_used = -3;
+	} else if (!isanumber(c_token)) {
 	    int_error(c_token, "Column number expected");
-	convert(&a, c_token++);
-	if (a.type != INTGR || a.v.int_val < 0)
-	    int_error(c_token, "Positive integer expected");
-	if (at_highest_column_used < a.v.int_val)
-	    at_highest_column_used = a.v.int_val;
+	} else {
+	    convert(&a, c_token++);
+	    if (a.type != INTGR || a.v.int_val < 0)
+		int_error(c_token, "Positive integer expected");
+	    if (at_highest_column_used < a.v.int_val)
+		at_highest_column_used = a.v.int_val;
+	}
 	add_action(DOLLARS)->v_arg = a;
     } else if (isanumber(c_token)) {
 	/* work around HP 9000S/300 HP-UX 9.10 cc limitation ... */
@@ -459,19 +491,20 @@ parse_primary_expression()
 	    struct value num_params;
 	    num_params.type = INTGR;
 
-	    if (whichfunc) {
-#ifdef BACKWARDS_COMPATIBLE
+#if (1)	    /* DEPRECATED */
+	    if (whichfunc && (strcmp(ft[whichfunc].f_name,"defined")==0)) {
 		/* Deprecated syntax:   if (defined(foo)) ...  */
 		/* New syntax:          if (exists("foo")) ... */
-		if (strcmp(ft[whichfunc].f_name,"defined")==0) {
-		    struct udvt_entry *udv = add_udv(c_token+2);
-		    union argument *foo = add_action(PUSHC);
-		    foo->v_arg.type = INTGR;
-		    foo->v_arg.v.int_val = udv->udv_undef ? 0 : 1;
-		    c_token += 4;  /* skip past "defined ( <foo> ) " */
-		    return;
-		}
+		struct udvt_entry *udv = add_udv(c_token+2);
+		union argument *foo = add_action(PUSHC);
+		foo->v_arg.type = INTGR;
+		foo->v_arg.v.int_val = udv->udv_undef ? 0 : 1;
+		c_token += 4;  /* skip past "defined ( <foo> ) " */
+		return;
+	    }
 #endif
+
+	    if (whichfunc) {
 		c_token += 2;	/* skip fnc name and '(' */
 		parse_expression(); /* parse fnc argument */
 		num_params.v.int_val = 1;
@@ -490,11 +523,9 @@ parse_primary_expression()
 		if (!strcmp(ft[whichfunc].f_name,"sprintf"))
 		    add_action(PUSHC)->v_arg = num_params;
 
-		/* "words(s)" is implemented as "word(s,-1)" */
-		if (!strcmp(ft[whichfunc].f_name,"words")) {
-		    num_params.v.int_val = -1;
+		/* v4 timecolumn only had 1 param; v5 has 2. Accept either */
+		if (!strcmp(ft[whichfunc].f_name,"timecolumn"))
 		    add_action(PUSHC)->v_arg = num_params;
-		}
 
 		/* The column() function has side effects requiring special handling */
 		if (!strcmp(ft[whichfunc].f_name,"column")) {
@@ -532,9 +563,11 @@ parse_primary_expression()
 	    if (equals(c_token, c_dummy_var[0])) {
 		c_token++;
 		add_action(PUSHD1)->udf_arg = dummy_func;
+		fit_dummy_var[0]++;
 	    } else if (equals(c_token, c_dummy_var[1])) {
 		c_token++;
 		add_action(PUSHD2)->udf_arg = dummy_func;
+		fit_dummy_var[1]++;
 	    } else {
 		int i, param = 0;
 
@@ -547,6 +580,7 @@ parse_primary_expression()
 			c_token++;
 			add_action(PUSHC)->v_arg = num_params;
 			add_action(PUSHD)->udf_arg = dummy_func;
+			fit_dummy_var[i]++;
 			break;
 		    }
 		}
@@ -590,7 +624,7 @@ parse_primary_expression()
      * Currently only used to generate substrings, but could also be used to
      * extract vector slices.
      */
-    if (equals(c_token, "[")) {
+    if (equals(c_token, "[") && !isanumber(c_token-1)) {
 	/* handle '*' or empty start of range */
 	if (equals(++c_token,"*") || equals(c_token,":")) {
 	    union argument *empty = add_action(PUSHC);
@@ -606,7 +640,7 @@ parse_primary_expression()
 	if (equals(++c_token,"*") || equals(c_token,"]")) {
 	    union argument *empty = add_action(PUSHC);
 	    empty->v_arg.type = INTGR;
-	    empty->v_arg.v.int_val = 65535; /* should be MAXINT */
+	    empty->v_arg.v.int_val = 65535; /* should be INT_MAX */
 	    if (equals(c_token,"*"))
 		c_token++;
 	} else
@@ -768,10 +802,9 @@ parse_relational_expression()
      * operators */
 
     while (TRUE) {
-	/* I hate "else if" statements */
 	if (equals(c_token, ">")) {
 	    c_token++;
-	    accept_additive_expression();
+	    accept_bitshift_expression();
 	    (void) add_action(GT);
 	} else if (equals(c_token, "<")) {
 	    /*  Workaround for * in syntax of range constraints  */
@@ -779,15 +812,15 @@ parse_relational_expression()
 		break;
 	    }
 	    c_token++;
-	    accept_additive_expression();
+	    accept_bitshift_expression();
 	    (void) add_action(LT);
 	} else if (equals(c_token, ">=")) {
 	    c_token++;
-	    accept_additive_expression();
+	    accept_bitshift_expression();
 	    (void) add_action(GE);
 	} else if (equals(c_token, "<=")) {
 	    c_token++;
-	    accept_additive_expression();
+	    accept_bitshift_expression();
 	    (void) add_action(LE);
 	} else
 	    break;
@@ -797,6 +830,26 @@ parse_relational_expression()
 
 
 
+static void
+parse_bitshift_expression()
+{
+    /* create action codes for << and >> operators */
+    while (TRUE) {
+	if (equals(c_token, "<<")) {
+	    c_token++;
+	    accept_additive_expression();
+	    (void) add_action(LEFTSHIFT);
+	} else if (equals(c_token, ">>")) {
+	    c_token++;
+	    accept_additive_expression();
+	    (void) add_action(RIGHTSHIFT);
+	} else
+	    break;
+    }
+}
+
+
+
 static void
 parse_additive_expression()
 {
@@ -862,9 +915,20 @@ parse_unary_expression()
 	parse_unary_expression();
 	(void) add_action(BNOT);
     } else if (equals(c_token, "-")) {
+	struct at_entry *previous;
 	c_token++;
 	parse_unary_expression();
-	(void) add_action(UMINUS);
+	/* Collapse two operations PUSHC <pos-const> + UMINUS
+	 * into a single operation PUSHC <neg-const>
+	 */
+	previous = &(at->actions[at->a_count-1]);
+	if (previous->index == PUSHC &&  previous->arg.v_arg.type == INTGR) {
+	    previous->arg.v_arg.v.int_val = -previous->arg.v_arg.v.int_val;
+	} else if (previous->index == PUSHC &&  previous->arg.v_arg.type == CMPLX) {
+	    previous->arg.v_arg.v.cmplx_val.real = -previous->arg.v_arg.v.cmplx_val.real;
+	    previous->arg.v_arg.v.cmplx_val.imag = -previous->arg.v_arg.v.cmplx_val.imag;
+	} else
+	    (void) add_action(UMINUS);
     } else if (equals(c_token, "+")) {	/* unary + is no-op */
 	c_token++;
 	parse_unary_expression();
@@ -873,6 +937,36 @@ parse_unary_expression()
 }
 
 
+/*
+ * Syntax: set link {x2|y2} {via <expression1> inverse <expression2>}
+ * Create action code tables for the functions linking primary and secondary axes.
+ * expression1 maps primary coordinates into the secondary coordinate space.
+ * expression2 maps secondary coordinates into the primary coordinate space.
+ */
+void
+parse_link_via( struct udft_entry *udf )
+{
+    int start_token;
+    
+    /* Caller left us pointing at "via" or "inverse" */
+    c_token++;
+    start_token = c_token;
+    if (END_OF_COMMAND)
+	int_error(c_token,"Missing expression");
+
+    /* Save action table for the linkage mapping */
+    strcpy(c_dummy_var[0], "x");
+    strcpy(c_dummy_var[1], "y");
+    dummy_func = udf;
+    free_at(udf->at);
+    udf->at = perm_at();
+    dummy_func = NULL;
+
+    /* Save the mapping expression itself */
+    m_capture(&(udf->definition), start_token, c_token - 1);
+}
+
+
 /* create action code for 'sum' expressions */
 static void
 parse_sum_expression()
@@ -1161,8 +1255,10 @@ next_iteration(t_iterator *iter)
 	    gpfree_string(&(this_iter->iteration_udv->udv_value));
 	    Gstring(&(this_iter->iteration_udv->udv_value), 
 		    gp_word(this_iter->iteration_string, this_iter->iteration_current));
-    } else
-	    this_iter->iteration_udv->udv_value.v.int_val = this_iter->iteration_current;	
+	} else {
+	    gpfree_string(&(this_iter->iteration_udv->udv_value));
+	    Ginteger(&(this_iter->iteration_udv->udv_value), this_iter->iteration_current);	
+	}
 	
 	this_iter = this_iter->prev;
     }
@@ -1179,8 +1275,11 @@ next_iteration(t_iterator *iter)
 	gpfree_string(&(this_iter->iteration_udv->udv_value));
 	Gstring(&(this_iter->iteration_udv->udv_value), 
 		gp_word(this_iter->iteration_string, this_iter->iteration_current));
-    } else
-	this_iter->iteration_udv->udv_value.v.int_val = this_iter->iteration_current;
+    } else {
+	/* This traps fatal user error of reassigning iteration variable to a string */
+	gpfree_string(&(this_iter->iteration_udv->udv_value));
+	Ginteger(&(this_iter->iteration_udv->udv_value), this_iter->iteration_current);	
+    }
     
     /* Mar 2014 revised to avoid integer overflow */
     if (this_iter->iteration_increment > 0
