@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: plot3d.c,v 1.191.2.8 2013/11/14 23:12:57 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: plot3d.c,v 1.231.2.4 2015/10/31 04:39:20 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - plot3d.c */
@@ -39,7 +39,6 @@ static char *RCSid() { return RCSid("$Id: plot3d.c,v 1.191.2.8 2013/11/14 23:12:
 
 #include "alloc.h"
 #include "axis.h"
-#include "binary.h"
 #include "command.h"
 #include "contour.h"
 #include "datafile.h"
@@ -62,6 +61,21 @@ static char *RCSid() { return RCSid("$Id: plot3d.c,v 1.191.2.8 2013/11/14 23:12:
 # include "help.h"
 #endif
 
+/* Sep 2013 - moved from axis.h in the process of doing away with it altogether */
+#if (1)	/* FIXME:  I'm 99% sure TRUE is fine, i.e. NEED_PALETTE is unnecessary. */
+	/* In any event, the alternative breaks splot with rgb variable */
+#define NEED_PALETTE(plot) TRUE
+#else
+#define NEED_PALETTE(plot) \
+    (  (pm3d.implicit == PM3D_IMPLICIT) \
+    || ((plot)->plot_style == PM3DSURFACE) \
+    || ((plot)->plot_style == IMAGE) \
+    || (plot)->lp_properties.pm3d_color.type == TC_CB \
+    || (plot)->lp_properties.pm3d_color.type == TC_FRAC \
+    || (plot)->lp_properties.pm3d_color.type == TC_Z \
+    )
+#endif
+
 /* global variables exported by this module */
 
 t_data_mapping mapping3d = MAP3D_CARTESIAN;
@@ -94,6 +108,9 @@ static void thin_plate_splines_setup __PROTO(( struct iso_curve *old_iso_crvs, d
 static double qnorm __PROTO(( double dist_x, double dist_y, int q ));
 static double pythag __PROTO(( double dx, double dy ));
 
+/* helper functions for parsing */
+static void load_contour_label_options __PROTO((struct text_label *contour_label));
+
 /* the curves/surfaces of the plot */
 struct surface_points *first_3dplot = NULL;
 static struct udft_entry plot_func;
@@ -180,7 +197,7 @@ sp_replace(
  *   surface points.
  */
 /* HBB 20000506: don't risk stack havoc by recursion, use iterative list
- * cleanup unstead */
+ * cleanup instead */
 void
 sp_free(struct surface_points *sp)
 {
@@ -215,9 +232,6 @@ sp_free(struct surface_points *sp)
 }
 
 
-
-/* support for dynamic size of input line */
-
 void
 plot3drequest()
 /*
@@ -232,10 +246,6 @@ plot3drequest()
 
     is_3d_plot = TRUE;
 
-    /* change view to become map if requested by 'set view map' */
-    if (splot_map == TRUE)
-	splot_map_activate();
-
     if (parametric && strcmp(set_dummy_var[0], "t") == 0) {
 	strcpy(set_dummy_var[0], "u");
 	strcpy(set_dummy_var[1], "v");
@@ -252,28 +262,24 @@ plot3drequest()
     if (!term)			/* unknown */
 	int_error(c_token, "use 'set term' to set terminal type first");
 
+    /* Range limits for the entire plot are optional but must be given	*/
+    /* in a fixed order. The keyword 'sample' terminates range parsing.	*/
     u_axis = (parametric ? U_AXIS : FIRST_X_AXIS);
     v_axis = (parametric ? V_AXIS : FIRST_Y_AXIS);
 
-    dummy_token0 = parse_named_range(u_axis, dummy_token0);
-    if (splot_map == TRUE && !parametric) /* v_axis==FIRST_Y_AXIS */
-	splot_map_deactivate();
-    dummy_token1 = parse_named_range(v_axis, dummy_token1);
-    if (splot_map == TRUE && !parametric) /* v_axis==FIRST_Y_AXIS */
-	splot_map_activate();
+    dummy_token0 = parse_range(u_axis);
+    dummy_token1 = parse_range(v_axis);
 
     if (parametric) {
 	parse_range(FIRST_X_AXIS);
-	if (splot_map == TRUE)
-	    splot_map_deactivate();
 	parse_range(FIRST_Y_AXIS);
-	if (splot_map == TRUE)
-	    splot_map_activate();
-    }				/* parametric */
+    }
     parse_range(FIRST_Z_AXIS);
     check_axis_reversed(FIRST_X_AXIS);
     check_axis_reversed(FIRST_Y_AXIS);
     check_axis_reversed(FIRST_Z_AXIS);
+    if (equals(c_token,"sample") && equals(c_token+1,"["))
+	c_token++;
 
     /* Clear out any tick labels read from data files in previous plot */
     for (u_axis=0; u_axis<AXIS_ARRAY_SIZE; u_axis++) {
@@ -285,21 +291,29 @@ plot3drequest()
     }
 
     /* use the default dummy variable unless changed */
-    if (dummy_token0 >= 0)
+    if (dummy_token0 > 0)
 	copy_str(c_dummy_var[0], dummy_token0, MAX_ID_LEN);
     else
-	(void) strcpy(c_dummy_var[0], set_dummy_var[0]);
+	strcpy(c_dummy_var[0], set_dummy_var[0]);
 
-    if (dummy_token1 >= 0)
+    if (dummy_token1 > 0)
 	copy_str(c_dummy_var[1], dummy_token1, MAX_ID_LEN);
     else
-	(void) strcpy(c_dummy_var[1], set_dummy_var[1]);
+	strcpy(c_dummy_var[1], set_dummy_var[1]);
+
+    /* In "set view map" mode the x2 and y2 axes are legal */
+    /* but must be linked to the respective primary axis. */
+    if (splot_map) {
+	if ((axis_array[SECOND_X_AXIS].ticmode && !axis_array[SECOND_X_AXIS].linked_to_primary)
+	||  (axis_array[SECOND_Y_AXIS].ticmode && !axis_array[SECOND_Y_AXIS].linked_to_primary))
+	    int_error(NO_CARET,
+		"Secondary axis must be linked to primary axis in order to draw tics");
+    }
 
     eval_3dplots();
 }
 
 
-#ifdef VOLATILE_REFRESH
 /* Helper function for refresh command.  Reexamine each data point and update the
  * flags for INRANGE/OUTRANGE/UNDEFINED based on the current limits for that axis.
  * Normally the axis limits are already known at this point. But if the user has
@@ -379,16 +393,20 @@ refresh_3dbounds(struct surface_points *first_plot, int nplots)
 		continue;
 	    }
 	}	/* End of this curve */
-	}	/* End of this plot */
+	}
 
-    }
+    }	/* End of this plot */
+
+    /* handle 'reverse' ranges */
+    axis_revert_range(FIRST_X_AXIS);
+    axis_revert_range(FIRST_Y_AXIS);
+    axis_revert_range(FIRST_Z_AXIS);
 
     /* Make sure the bounds are reasonable, and tweak them if they aren't */
     axis_checked_extend_empty_range(FIRST_X_AXIS, NULL);
     axis_checked_extend_empty_range(FIRST_Y_AXIS, NULL);
     axis_checked_extend_empty_range(FIRST_Z_AXIS, NULL);
 }
-#endif
 
 
 static double
@@ -541,7 +559,8 @@ grid_nongrid_data(struct surface_points *this_plot)
     
     /* these are only needed for thin_plate_splines */
     double *xx, *yy, *zz, *b;
-    int numpoints;
+    int numpoints = 0;
+
     xx = NULL; /* save to call free() on NULL if xx has never been used */
     
     /* Compute XY bounding box on the original data. */
@@ -679,7 +698,7 @@ grid_nongrid_data(struct surface_points *this_plot)
 	    points->y = y;
 
 	    /* Honor requested x and y limits */
-	    /* FIXME: This code section was not in 4.2.  It imperfectly    */
+	    /* Historical note: This code was not in 4.0 or 4.2. It imperfectly */
 	    /* restores the clipping behaviour of version 3.7 and earlier. */
 	    if ((x < axis_array[x_axis].min && !(axis_array[x_axis].autoscale & AUTOSCALE_MIN))
 	    ||  (x > axis_array[x_axis].max && !(axis_array[x_axis].autoscale & AUTOSCALE_MAX))
@@ -775,7 +794,7 @@ get_3ddata(struct surface_points *this_plot)
 	/*{{{  read surface from text file */
 	struct iso_curve *local_this_iso = iso_alloc(samples_1);
 	struct coordinate GPHUGE *cp;
-	struct coordinate GPHUGE *cptail = NULL; /* Only for VECTOR plots */
+	struct coordinate GPHUGE *cphead = NULL; /* Only for VECTOR plots */
 	double x, y, z;
 	double xtail, ytail, ztail;
 	double color = VERYLARGE;
@@ -880,7 +899,7 @@ get_3ddata(struct surface_points *this_plot)
 		    cp->type = UNDEFINED;
 		    continue;
 		}
-		cptail = local_this_iso->next->points + xdatum;
+		cphead = local_this_iso->next->points + xdatum;
 	    }
 
 	    if (j == DF_UNDEFINED || j == DF_MISSING) {
@@ -976,8 +995,8 @@ get_3ddata(struct surface_points *this_plot)
 			"Wrong number of columns in input data - line %d",
 			df_line_number);
 
-	    /* FIXME: Work-around for hidden3d, which otherwise would use */
-	    /* the color of the vector midpoint rather than the endpoint. */
+	    /* Work-around for hidden3d, which otherwise would use the */
+	    /* color of the vector midpoint rather than the endpoint. */
 	    if (this_plot->plot_style == IMPULSES) {
 		if (this_plot->lp_properties.pm3d_color.type == TC_Z) {
 		    color = z;
@@ -1050,9 +1069,9 @@ get_3ddata(struct surface_points *this_plot)
 	    STORE_WITH_LOG_AND_UPDATE_RANGE(cp->x, x, cp->type, x_axis, this_plot->noautoscale, NOOP, goto come_here_if_undefined);
 	    STORE_WITH_LOG_AND_UPDATE_RANGE(cp->y, y, cp->type, y_axis, this_plot->noautoscale, NOOP, goto come_here_if_undefined);
 	    if (this_plot->plot_style == VECTOR) {
-		cptail->type = INRANGE;
-		STORE_WITH_LOG_AND_UPDATE_RANGE(cptail->x, xtail, cp->type, x_axis, this_plot->noautoscale, NOOP, goto come_here_if_undefined);
-		STORE_WITH_LOG_AND_UPDATE_RANGE(cptail->y, ytail, cp->type, y_axis, this_plot->noautoscale, NOOP, goto come_here_if_undefined);
+		cphead->type = INRANGE;
+		STORE_WITH_LOG_AND_UPDATE_RANGE(cphead->x, xtail, cphead->type, x_axis, this_plot->noautoscale, NOOP, goto come_here_if_undefined);
+		STORE_WITH_LOG_AND_UPDATE_RANGE(cphead->y, ytail, cphead->type, y_axis, this_plot->noautoscale, NOOP, goto come_here_if_undefined);
 	    }
 
 	    if (dgrid3d) {
@@ -1062,17 +1081,22 @@ get_3ddata(struct surface_points *this_plot)
 		 * created created grid more easily. */
 		cp->z = z;
 		if (this_plot->plot_style == VECTOR)
-		    cptail->z = ztail;
+		    cphead->z = ztail;
 	    } else {
-		/* EAM Sep 2008 - Otherwise z=Nan or z=Inf or DF_MISSING fails */
-		/* to set CRD_COLOR at all, since the z test bails to a goto.  */
+
+		/* EAM Sep 2008 - Otherwise z=Nan or z=Inf or DF_MISSING fails	*/
+		/* to set CRD_COLOR at all, since the z test bails to a goto. 	*/
 		if (this_plot->plot_style == IMAGE) {
-			cp->CRD_COLOR = z;
+			cp->CRD_COLOR = (pm3d_color_from_column) ? color : z;
 	        }
 
-		STORE_WITH_LOG_AND_UPDATE_RANGE(cp->z, z, cp->type, z_axis, this_plot->noautoscale, NOOP, goto come_here_if_undefined);
+		/* Version 5: cp->z=0 in the UNDEF_ACTION recovers what	version 4 did */
+		STORE_WITH_LOG_AND_UPDATE_RANGE(cp->z, z, cp->type, z_axis,
+				this_plot->noautoscale, NOOP,
+				cp->z=0;goto come_here_if_undefined);
+
 		if (this_plot->plot_style == VECTOR)
-		    STORE_WITH_LOG_AND_UPDATE_RANGE(cptail->z, ztail, cp->type, z_axis, this_plot->noautoscale, NOOP, goto come_here_if_undefined);
+		    STORE_WITH_LOG_AND_UPDATE_RANGE(cphead->z, ztail, cphead->type, z_axis, this_plot->noautoscale, NOOP, goto come_here_if_undefined);
 
 		if (this_plot->lp_properties.l_type == LT_COLORFROMCOLUMN)
 		    cp->CRD_COLOR = color;
@@ -1150,7 +1174,9 @@ get_3ddata(struct surface_points *this_plot)
 
     if (this_plot->num_iso_read <= 1)
 	this_plot->has_grid_topology = FALSE;
-    if (this_plot->has_grid_topology && !hidden3d) {
+
+    if (this_plot->has_grid_topology && !hidden3d 
+    &&   (implicit_surface || this_plot->plot_style == SURFACEGRID)) {
 	struct iso_curve *new_icrvs = NULL;
 	int num_new_iso = this_plot->iso_crvs->p_count;
 	int len_new_iso = this_plot->num_iso_read;
@@ -1273,7 +1299,7 @@ eval_3dplots()
     TBOOLEAN some_data_files = FALSE, some_functions = FALSE;
     TBOOLEAN was_definition = FALSE;
     int df_return = 0;
-    int plot_num, line_num, point_num;
+    int plot_num, line_num;
     /* part number of parametric function triplet: 0 = z, 1 = y, 2 = x */
     int crnt_param = 0;
     char *xtitle;
@@ -1296,7 +1322,6 @@ eval_3dplots()
     tp_3d_ptr = &(first_3dplot);
     plot_num = 0;
     line_num = 0;		/* default line type */
-    point_num = 0;		/* default point type */
 
     /* Assume that the input data can be re-read later */
     volatile_data = FALSE;
@@ -1328,10 +1353,10 @@ eval_3dplots()
 
 	if (is_definition(c_token)) {
 	    define();
-	    if (!equals(c_token,",")) {
-		was_definition = TRUE;
-		continue;
-	    }
+	    if (equals(c_token, ","))
+		c_token++;
+	    was_definition = TRUE;
+	    continue;
 
 	} else {
 	    int specs = -1;
@@ -1343,14 +1368,19 @@ eval_3dplots()
 	    TBOOLEAN set_lpstyle = FALSE;
 	    TBOOLEAN checked_once = FALSE;
 	    TBOOLEAN set_labelstyle = FALSE;
+	    int sample_range_token;
 
 	    if (!was_definition && (!parametric || crnt_param == 0))
 		start_token = c_token;
 	    was_definition = FALSE;
 
+	    /* Check for a sampling range */
+	    sample_range_token = parse_range(SAMPLE_AXIS);
+	    if (sample_range_token > 0)
+		axis_array[SAMPLE_AXIS].range_flags |= RANGE_SAMPLED;
+
+	    /* Should this be saved in this_plot? */
 	    dummy_func = &plot_func;
-	    /* WARNING: do NOT free name_str */
-	    /* FIXME: could also be saved in this_plot */
 	    name_str = string_or_express(NULL);
 	    dummy_func = NULL;
 	    if (name_str) {
@@ -1390,8 +1420,21 @@ eval_3dplots()
 		if (df_matrix)
 		    this_plot->has_grid_topology = TRUE;
 
-		/* parses all datafile-specific modifiers */
-		/* we will load the data after parsing title,with,... */
+		/* EAM FIXME - this seems to work but I am uneasy that c_dummy_var[]	*/
+		/*             is not being loaded with the variable name.		*/
+		if (sample_range_token > 0) {
+		    this_plot->sample_var = add_udv(sample_range_token);
+		    this_plot->sample_var->udv_undef = FALSE;
+		} else {
+		    /* FIXME: This has the side effect of creating a named variable x */
+		    /* or overwriting an existing variable x.  Maybe it should save   */
+		    /* and restore the pre-existing variable in this case?            */
+		    this_plot->sample_var = add_udv_by_name(c_dummy_var[0]);
+		    if (this_plot->sample_var->udv_undef) {
+			this_plot->sample_var->udv_undef = FALSE;
+			Gcomplex(&(this_plot->sample_var->udv_value), 0.0, 0.0);
+		    }
+		}
 
 		/* for capture to key */
 		this_plot->token = end_token = c_token - 1;
@@ -1460,10 +1503,10 @@ eval_3dplots()
 		this_plot->title = NULL;
 	    }
 
-	    /* default line and point types, no palette */
-	    this_plot->lp_properties.use_palette = 0;
+	    /* default line and point types */
 	    this_plot->lp_properties.l_type = line_num;
-	    this_plot->lp_properties.p_type = point_num;
+	    this_plot->lp_properties.p_type = line_num;
+		this_plot->lp_properties.d_type = line_num;
 
 	    /* user may prefer explicit line styles */
 	    this_plot->hidden3d_top_linetype = line_num;
@@ -1476,57 +1519,36 @@ eval_3dplots()
 	    while (!END_OF_COMMAND || !checked_once) {
 
 		/* deal with title */
-		if (almost_equals(c_token, "t$itle")) {
+		if (almost_equals(c_token, "t$itle") || almost_equals(c_token, "not$itle")) {
 		    if (set_title) {
 			duplication=TRUE;
 			break;
 		    }
-		    this_plot->title_no_enhanced = !key->enhanced;
-			/* title can be enhanced if not explicitly disabled */
-		    if (parametric) {
-			if (crnt_param != 0)
-			    int_error(c_token, "\"title\" allowed only after parametric function fully specified");
-			else {
-			    if (xtitle != NULL)
-				xtitle[0] = NUL;	/* Remove default title . */
-			    if (ytitle != NULL)
-				ytitle[0] = NUL;	/* Remove default title . */
-			}
+		    set_title = TRUE;
+		    if (almost_equals(c_token++, "not$itle")) {
+			this_plot->title_is_suppressed = TRUE;
+			if (equals(c_token,","))
+			    continue;
 		    }
-		    c_token++;
+
+		    if (parametric || this_plot->title_is_suppressed) {
+			if (xtitle != NULL)
+			    xtitle[0] = NUL;	/* Remove default title . */
+			if (ytitle != NULL)
+			    ytitle[0] = NUL;	/* Remove default title . */
+		    }
+
+		    /* title can be enhanced if not explicitly disabled */
+		    this_plot->title_no_enhanced = !key->enhanced;
 
 		    if (almost_equals(c_token,"col$umnheader")) {
 			df_set_key_title_columnhead((struct curve_points *)this_plot);
-		    } else 
-
-#ifdef BACKWARDS_COMPATIBLE
-		    /* Annoying backwards-compatibility hack - deprecate! */
-		    if (isanumber(c_token)) {
-			c_token--;
-			df_set_key_title_columnhead((struct curve_points *)this_plot);
-		    } else
-#endif
-
-		    if (!(this_plot->title = try_to_get_string()))
-			int_error(c_token, "expecting \"title\" for plot");
-		    set_title = TRUE;
-		    continue;
-		}
-
-		if (almost_equals(c_token, "not$itle")) {
-		    if (set_title) {
-			duplication=TRUE;
-			break;
+		    } else {
+			char *temp;
+			temp = try_to_get_string();
+			if (!this_plot->title_is_suppressed && !(this_plot->title = temp))
+			    int_error(c_token, "expecting \"title\" for plot");
 		    }
-		    c_token++;
-		    if (isstringvalue(c_token))
-			try_to_get_string(); /* ignore optionally given title string */
-		    this_plot->title_is_suppressed = TRUE;
-		    if (xtitle != NULL)
-			xtitle[0] = '\0';
-		    if (ytitle != NULL)
-			ytitle[0] = '\0';
-		    set_title = TRUE;
 		    continue;
 		}
 
@@ -1549,9 +1571,20 @@ eval_3dplots()
 		    this_plot->plot_style = get_style();
 		    if ((this_plot->plot_type == FUNC3D) &&
 			((this_plot->plot_style & PLOT_STYLE_HAS_ERRORBAR)
-			|| (this_plot->plot_style == LABELPOINTS))) {
-			int_warn(c_token, "This plot style is only for datafiles , reverting to \"points\"");
+			|| (this_plot->plot_style == LABELPOINTS && !draw_contour)
+			)) {
+			int_warn(c_token-1, "This style cannot be used to plot a surface defined by a function");
 			this_plot->plot_style = POINTSTYLE;
+			this_plot->plot_type = NODATA;
+		    }
+
+		    if (this_plot->plot_style == IMAGE
+		    ||  this_plot->plot_style == RGBA_IMAGE
+		    ||  this_plot->plot_style == RGBIMAGE) {
+			if (this_plot->plot_type == FUNC3D)
+			    int_error(c_token-1, "a function cannot be plotted as an image");
+			else
+			    get_image_options(&this_plot->image_properties);
 		    }
 
 		    if ((this_plot->plot_style | data_style) & PM3DSURFACE) {
@@ -1563,10 +1596,8 @@ eval_3dplots()
 			}
 		    }
 
-		    if (this_plot->plot_style == IMAGE
-		    ||  this_plot->plot_style == RGBA_IMAGE
-		    ||  this_plot->plot_style == RGBIMAGE)
-			get_image_options(&this_plot->image_properties);
+		    if (this_plot->plot_style == TABLESTYLE)
+			int_error(NO_CARET, "use `plot with table` rather than `splot with table`"); 
 
 		    set_with = TRUE;
 		    continue;
@@ -1595,33 +1626,8 @@ eval_3dplots()
 		    continue;
 		}
 
-		/* Labels can have font and text property info as plot options */
-		/* In any case we must allocate one instance of the text style */
-		/* that all labels in the plot will share.                     */
-		if (this_plot->plot_style == LABELPOINTS)  {
-		    int stored_token = c_token;
-		    if (this_plot->labels == NULL) {
-			this_plot->labels = new_text_label(-1);
-			this_plot->labels->pos = CENTRE;
-			this_plot->labels->layer = LAYER_PLOTLABELS;
-		    }
-		    parse_label_options(this_plot->labels);
-		    checked_once = TRUE;
-		    if (stored_token != c_token) {
-			if (set_labelstyle) {
-			    duplication = TRUE;
-			    break;
-			} else {
-			    set_labelstyle = TRUE;
-			    continue;
-			}
-		    }
-		}
-
-		/* pick up line/point specs
-		 * - point spec allowed if style uses points, ie style&2 != 0
-		 * - keywords are optional
-		 */
+		/* Most plot styles accept line and point properties but do not */
+		/* want font or text properties.				*/
 		if (this_plot->plot_style == VECTOR) {
 		    int stored_token = c_token;
 
@@ -1642,13 +1648,30 @@ eval_3dplots()
 			    continue;
 			}
 		    }
-		} else {
+
+		}
+
+		if (this_plot->plot_style == PM3DSURFACE) {
+		    /* both previous and subsequent line properties override pm3d default border */
+		    int stored_token = c_token;
+		    if (!set_lpstyle)
+			this_plot->lp_properties = pm3d.border;
+ 		    lp_parse(&this_plot->lp_properties, LP_ADHOC, FALSE);
+		    if (stored_token != c_token) {
+			set_lpstyle = TRUE;
+			continue;
+		    }
+
+		}
+
+		if (this_plot->plot_style != LABELPOINTS) {
 		    int stored_token = c_token;
 		    struct lp_style_type lp = DEFAULT_LP_STYLE_TYPE;
 		    int new_lt = 0;
 
 		    lp.l_type = line_num;
-		    lp.p_type = point_num;
+		    lp.p_type = line_num;
+		    lp.d_type = line_num;
 
 		    /* user may prefer explicit line styles */
 		    if (prefer_line_styles)
@@ -1656,7 +1679,7 @@ eval_3dplots()
 		    else
 			load_linetype(&lp, line_num+1);
 
- 		    new_lt = lp_parse(&lp, TRUE,
+ 		    new_lt = lp_parse(&lp, LP_ADHOC,
 			     this_plot->plot_style & PLOT_STYLE_HAS_POINT);
 
 		    checked_once = TRUE;
@@ -1669,8 +1692,34 @@ eval_3dplots()
 			    set_lpstyle = TRUE;
 			    if (new_lt)
 				this_plot->hidden3d_top_linetype = new_lt - 1;
-			    else
-				this_plot->hidden3d_top_linetype = line_num;
+			    if (this_plot->lp_properties.p_type != PT_CHARACTER)
+				continue;
+			}
+		    }
+		}
+
+		/* Labels can have font and text property info as plot options */
+		/* In any case we must allocate one instance of the text style */
+		/* that all labels in the plot will share.                     */
+		if ((this_plot->plot_style == LABELPOINTS)
+		||  (this_plot->plot_style & PLOT_STYLE_HAS_POINT
+			&& this_plot->lp_properties.p_type == PT_CHARACTER)) {
+		    int stored_token = c_token;
+		    if (this_plot->labels == NULL) {
+			this_plot->labels = new_text_label(-1);
+			this_plot->labels->pos = CENTRE;
+			this_plot->labels->layer = LAYER_PLOTLABELS;
+		    }
+		    parse_label_options(this_plot->labels, TRUE);
+		    if (draw_contour)
+			load_contour_label_options(this_plot->labels);
+		    checked_once = TRUE;
+		    if (stored_token != c_token) {
+			if (set_labelstyle) {
+			    duplication = TRUE;
+			    break;
+			} else {
+			    set_labelstyle = TRUE;
 			    continue;
 			}
 		    }
@@ -1711,13 +1760,19 @@ eval_3dplots()
 		    this_plot->arrow_properties.lp_properties.l_type = line_num;
 		    arrow_parse(&this_plot->arrow_properties, TRUE);
 		    this_plot->lp_properties = this_plot->arrow_properties.lp_properties;
+
+		} else if (this_plot->plot_style == PM3DSURFACE) {
+		    /* Use default pm3d border unless we see explicit line properties */
+		    this_plot->lp_properties = pm3d.border;
+ 		    lp_parse(&this_plot->lp_properties, LP_ADHOC, FALSE);
+
 		} else {
 		    int new_lt = 0;
 		    this_plot->lp_properties.l_type = line_num;
 		    this_plot->lp_properties.l_width = 1.0;
-		    this_plot->lp_properties.p_type = point_num;
+		    this_plot->lp_properties.p_type = line_num;
+			this_plot->lp_properties.d_type = line_num;
 		    this_plot->lp_properties.p_size = pointsize;
-		    this_plot->lp_properties.use_palette = 0;
 
 		    /* user may prefer explicit line styles */
 		    if (prefer_line_styles)
@@ -1725,31 +1780,19 @@ eval_3dplots()
 		    else
 			load_linetype(&this_plot->lp_properties, line_num+1);
 
-		    new_lt = lp_parse(&this_plot->lp_properties, TRUE,
+		    new_lt = lp_parse(&this_plot->lp_properties, LP_ADHOC,
 			 this_plot->plot_style & PLOT_STYLE_HAS_POINT);
 		    if (new_lt)
 			this_plot->hidden3d_top_linetype = new_lt - 1;
 		    else
 			this_plot->hidden3d_top_linetype = line_num;
 		}
-
-#ifdef BACKWARDS_COMPATIBLE
-		/* allow old-style syntax - ignore case lt 3 4 for example */
-		if (!END_OF_COMMAND && isanumber(c_token)) {
-		    this_plot->lp_properties.l_type =
-			this_plot->lp_properties.p_type = int_expression() - 1;
-
-		    if (isanumber(c_token))
-			this_plot->lp_properties.p_type = int_expression() - 1;
-		}
-#endif
-
 	    }
 
 	    /* Some low-level routines expect to find the pointflag attribute */
 	    /* in lp_properties (they don't have access to the full header).  */
 	    if (this_plot->plot_style & PLOT_STYLE_HAS_POINT)
-		this_plot->lp_properties.pointflag = TRUE;
+		this_plot->lp_properties.flags |= LP_SHOW_POINTS;
 
 	    /* Rule out incompatible line/point/style options */
 	    if (this_plot->plot_type == FUNC3D) {
@@ -1770,13 +1813,9 @@ eval_3dplots()
 		&& this_plot->plot_style != RGBA_IMAGE
 		/* same as above, for an (rgb)image plot */
 		) {
-		if (this_plot->plot_style & PLOT_STYLE_HAS_POINT)
-		    point_num += 1 + (draw_contour != 0) + (hidden3d != 0);
 		line_num += 1 + (draw_contour != 0) + (hidden3d != 0);
 	    }
 
-	    if (this_plot->plot_style == IMAGE)
-		this_plot->lp_properties.use_palette = 1;
 	    if (this_plot->plot_style == RGBIMAGE || this_plot->plot_style == RGBA_IMAGE) {
 		if (CB_AXIS.autoscale & AUTOSCALE_MIN)
 		    CB_AXIS.min = 0;
@@ -1855,7 +1894,9 @@ eval_3dplots()
 		    this_plot->iteration = plot_iterator ? plot_iterator->iteration : 0;
 		    this_plot->plot_style = first_dataset->plot_style;
 		    this_plot->lp_properties = first_dataset->lp_properties;
-		    if (this_plot->plot_style == LABELPOINTS) {
+		    if ((this_plot->plot_style == LABELPOINTS)
+		    ||  (this_plot->plot_style & PLOT_STYLE_HAS_POINT
+			    && this_plot->lp_properties.p_type == PT_CHARACTER)) {
 			this_plot->labels = new_text_label(-1);
 			*(this_plot->labels) = *(first_dataset->labels);
 			this_plot->labels->next = NULL;
@@ -1863,6 +1904,19 @@ eval_3dplots()
 		} while (df_return != DF_EOF);
 
 		df_close();
+
+		/* Plot-type specific range-fiddling */
+		if (this_plot->plot_style == IMPULSES && !axis_array[FIRST_Z_AXIS].log) {
+		    if (axis_array[FIRST_Z_AXIS].autoscale & AUTOSCALE_MIN) {
+			if (axis_array[FIRST_Z_AXIS].min > 0)
+			    axis_array[FIRST_Z_AXIS].min = 0;
+		    }
+		    if (axis_array[FIRST_Z_AXIS].autoscale & AUTOSCALE_MAX) {
+			if (axis_array[FIRST_Z_AXIS].max < 0)
+			    axis_array[FIRST_Z_AXIS].max = 0;
+		    }
+		}
+
 		/*}}} */
 
 	    } else {		/* not a data file */
@@ -1972,12 +2026,6 @@ eval_3dplots()
 	c_token = begin_token;
 	plot_iterator = check_for_iteration();
 
-	/* why do attributes of this_plot matter ? */
-	/* FIXME HBB 20000501: I think they don't, actually. I'm
-	 * taking out references to has_grid_topology in this part of
-	 * the code.  It only deals with function, which always is
-	 * gridded */
-
 	if (hidden3d) {
 	    u_step = (u_max - u_min) / (iso_samples_1 - 1);
 	    v_step = (v_max - v_min) / (iso_samples_2 - 1);
@@ -1997,10 +2045,10 @@ eval_3dplots()
 
 	    if (is_definition(c_token)) {
 		define();
-		if (!equals(c_token,",")) {
-		    was_definition = TRUE;
-		    continue;
-		}
+		if (equals(c_token,","))
+		    c_token++;
+		was_definition = TRUE;
+		continue;
 
 	    } else {
 		struct at_type *at_ptr;
@@ -2013,6 +2061,11 @@ eval_3dplots()
 		    break;
 		}
 
+		/* Check for a sampling range */
+		/* Currently we are supporting only sampling of pseudofile '+' and   */
+		/* this loop is for functions only, so the sampling range is ignored */
+		parse_range(SAMPLE_AXIS);
+
 		dummy_func = &plot_func;
 		name_str = string_or_express(&at_ptr);
 
@@ -2115,6 +2168,10 @@ eval_3dplots()
     setup_tics(FIRST_X_AXIS, 20);
     setup_tics(FIRST_Y_AXIS, 20);
     setup_tics(FIRST_Z_AXIS, 20);
+    if (splot_map) {
+	setup_tics(SECOND_X_AXIS, 20);
+	setup_tics(SECOND_Y_AXIS, 20);
+    }
 
     set_plot_with_palette(plot_num, MODE_SPLOT);
     if (is_plot_with_palette()) {
@@ -2151,8 +2208,7 @@ eval_3dplots()
 	    }
 
 	    /* Make sure this one can be contoured. */
-	    if (this_plot->plot_style == LABELPOINTS
-	    ||  this_plot->plot_style == VECTOR
+	    if (this_plot->plot_style == VECTOR
 	    ||  this_plot->plot_style == IMAGE
 	    ||  this_plot->plot_style == RGBIMAGE
 	    ||  this_plot->plot_style == RGBA_IMAGE)
@@ -2188,8 +2244,7 @@ eval_3dplots()
 	plot_token = -1;
 	fill_gpval_string("GPVAL_LAST_PLOT", replot_line);
     }
-
-    /* record that all went well */
+/* record that all went well */
     plot3d_num=plot_num;
 
     /* perform the plot */
@@ -2207,11 +2262,8 @@ eval_3dplots()
 	/* update GPVAL_ variables available to user */
 	update_gpval_variables(1);
 
-#ifdef VOLATILE_REFRESH
 	/* Mark these plots as safe for quick refresh */
-	refresh_nplots = plot_num;
-	refresh_ok = 3;
-#endif
+	SET_REFRESH_OK(E_REFRESH_OK_3D, plot_num);
     }
 }
 
@@ -2307,3 +2359,12 @@ parametric_3dfixup(struct surface_points *start_plot, int *plot_num)
     first_3dplot = new_list;
 }
 
+static void load_contour_label_options (struct text_label *contour_label)
+{
+    struct lp_style_type *lp = &(contour_label->lp_properties);
+    if (!contour_label->font)
+	contour_label->font = gp_strdup(clabel_font);
+    lp->p_interval = clabel_interval;
+    lp->flags |= LP_SHOW_POINTS;
+    lp_parse(lp, LP_ADHOC, TRUE);
+}
