@@ -1,5 +1,5 @@
 /*
- * $Id: winmain.c,v 1.52.2.8 2014/07/04 09:31:06 markisch Exp $
+ * $Id: winmain.c,v 1.77.2.2 2014/12/31 04:38:44 sfeam Exp $
  */
 
 /* GNUPLOT - win/winmain.c */
@@ -49,21 +49,14 @@
 /* and Russell Lang (rjl@monu1.cc.monash.edu.au) 30 Nov 1992          */
 /*                                                                    */
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
+#include "syscfg.h"
 #define STRICT
-/* required for MinGW64 */
-#define _WIN32_IE 0x0501
-#define _WIN32_WINNT 0x0501
 #include <windows.h>
 #include <windowsx.h>
 #include <commctrl.h>
 #include <shlobj.h>
 #include <shlwapi.h>
-#ifdef WITH_HTML_HELP
 #include <htmlhelp.h>
-#endif
 #include <dos.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -71,14 +64,12 @@
 #include <stdarg.h>
 #include <ctype.h>
 #include <fcntl.h>
-#ifdef __MSC__
-# include <malloc.h>
-#endif
 #ifdef __WATCOMC__
 # define mktemp _mktemp
 #endif
 #include <io.h>
 #include <sys/stat.h>
+#include "alloc.h"
 #include "plot.h"
 #include "setshow.h"
 #include "version.h"
@@ -92,6 +83,11 @@
 #ifdef WXWIDGETS
 #include "wxterminal/wxt_term.h"
 #endif
+#ifdef HAVE_LIBCACA
+# define TERM_PUBLIC_PROTO
+# include "caca.trm"
+# undef TERM_PUBLIC_PROTO
+#endif
 
 
 /* workaround for old header files */
@@ -122,9 +118,7 @@ BOOL cp_changed = FALSE;
 UINT cp_input;  /* save previous codepage settings */
 UINT cp_output;
 #endif
-#ifdef WITH_HTML_HELP
 HWND help_window = NULL;
-#endif
 
 char *authors[]={
                  "Colin Kelley",
@@ -141,10 +135,11 @@ CheckMemory(LPSTR str)
 {
         if (str == (LPSTR)NULL) {
                 MessageBox(NULL, "out of memory", "gnuplot", MB_ICONSTOP | MB_OK);
-                exit(1);
+                gp_exit(EXIT_FAILURE);
         }
 }
 
+
 int
 Pause(LPSTR str)
 {
@@ -152,16 +147,18 @@ Pause(LPSTR str)
         return (PauseBox(&pausewin) == IDOK);
 }
 
+
 void
 kill_pending_Pause_dialog ()
 {
-        if (pausewin.bPause == FALSE) /* no Pause dialog displayed */
+	if (!pausewin.bPause) /* no Pause dialog displayed */
             return;
         /* Pause dialog displayed, thus kill it */
         DestroyWindow(pausewin.hWndPause);
         pausewin.bPause = FALSE;
 }
 
+
 /* atexit procedure */
 void
 WinExit(void)
@@ -185,14 +182,11 @@ WinExit(void)
 
 #ifndef WGP_CONSOLE
     TextMessage();  /* process messages */
-#ifndef WITH_HTML_HELP
-    WinHelp(textwin.hWndText,(LPSTR)winhelpname,HELP_QUIT,(DWORD)NULL);
-#endif
-    TextMessage();  /* process messages */
+# ifndef __WATCOMC__
+    /* revert C++ stream redirection */
+    RedirectOutputStreams(FALSE);
+# endif
 #else
-#ifndef WITH_HTML_HELP
-    WinHelp(GetDesktopWindow(), (LPSTR)winhelpname, HELP_QUIT, (DWORD)NULL);
-#endif
 #ifdef CONSOLE_SWITCH_CP
     /* restore console codepages */
     if (cp_changed) {
@@ -214,7 +208,7 @@ ShutDown()
 {
 	/* First chance for wgnuplot to close help system. */
 	WinCloseHelp();
-	exit(0);
+	gp_exit(EXIT_SUCCESS);
 	return 0;
 }
 
@@ -310,7 +304,6 @@ appdata_directory(void)
 static void
 WinCloseHelp(void)
 {
-#ifdef WITH_HTML_HELP
 	/* Due to a known bug in the HTML help system we have to
 	 * call this as soon as possible before the end of the program.
 	 * See e.g. http://helpware.net/FAR/far_faq.htm#HH_CLOSE_ALL
@@ -318,7 +311,6 @@ WinCloseHelp(void)
 	if (IsWindow(help_window))
 		SendMessage(help_window, WM_CLOSE, 0, 0);
 	Sleep(0);
-#endif
 }
 
 
@@ -334,8 +326,8 @@ GetLanguageCode()
 		if (strcmp(lang, "JPN") == 0)
 			lang[1] = 'A';
 		/* prefer lower case */
-		lang[0] = tolower(lang[0]);
-		lang[1] = tolower(lang[1]);
+		lang[0] = tolower((unsigned char)lang[0]);
+		lang[1] = tolower((unsigned char)lang[1]);
 		/* only use two character sequence */
 		lang[2] = NUL;
 	}
@@ -344,14 +336,6 @@ GetLanguageCode()
 }
 
 
-static BOOL
-FileExists(const char * filename)
-{
-	struct stat buffer;
-	return stat(filename, &buffer) == 0;
-}
-
-
 static char *
 LocalisedFile(const char * name, const char * ext, const char * defaultname)
 {
@@ -370,7 +354,7 @@ LocalisedFile(const char * name, const char * ext, const char * defaultname)
 		strcat(filename, name);
 		strcat(filename, lang);
 		strcat(filename, ext);
-		if (!FileExists(filename)) {
+		if (!existfile(filename)) {
 			strcpy(filename, szModuleName);
 			strcat(filename, defaultname);
 		}
@@ -382,12 +366,8 @@ LocalisedFile(const char * name, const char * ext, const char * defaultname)
 static void
 ReadMainIni(LPSTR file, LPSTR section)
 {
-    char profile[81] = "";
-#ifdef WITH_HTML_HELP
+	char profile[81] = "";
 	const char hlpext[] = ".chm";
-#else
-	const char hlpext[] = ".hlp";
-#endif
 	const char name[] = "wgnuplot-";
 
 	/* Language code override */
@@ -426,10 +406,11 @@ ReadMainIni(LPSTR file, LPSTR section)
 
 
 #ifndef WGP_CONSOLE
-int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
-                LPSTR lpszCmdLine, int nCmdShow)
+int WINAPI
+WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int nCmdShow)
 #else
-int main(int argc, char **argv)
+int
+main(int argc, char **argv)
 #endif
 {
 	LPSTR tail;
@@ -441,7 +422,7 @@ int main(int argc, char **argv)
 
 
 #ifndef WGP_CONSOLE
-# if defined (__MINGW32__) && !defined(_W64)
+# if defined( __MINGW32__) && !defined(_W64)
 #  define argc _argc
 #  define argv _argv
 # else /* MSVC, WATCOM, MINGW-W64 */
@@ -482,7 +463,7 @@ int main(int argc, char **argv)
 #endif
 
 		/* create structure of first graph window */
-		graphwin = calloc(1, sizeof(GW));
+		graphwin = (LPGW) calloc(1, sizeof(GW));
 		listgraphs = graphwin;
 
 		/* locate ini file */
@@ -554,10 +535,11 @@ int main(int argc, char **argv)
 	}
 
 #ifndef WGP_CONSOLE
-        if (TextInit(&textwin))
-                exit(1);
-        textwin.hIcon = LoadIcon(hInstance, "TEXTICON");
-        SetClassLongPtr(textwin.hWndParent, GCLP_HICON, (LONG_PTR)textwin.hIcon);
+	if (TextInit(&textwin))
+		gp_exit(EXIT_FAILURE);
+	textwin.hIcon = LoadIcon(hInstance, "TEXTICON");
+	SetClassLongPtr(textwin.hWndParent, GCLP_HICON, (LONG_PTR)textwin.hIcon);
+
 	/* Note: we want to know whether this is an interactive session so that we can
 	 * decide whether or not to write status information to stderr.  The old test
 	 * for this was to see if (argc > 1) but the addition of optional command line
@@ -573,15 +555,18 @@ int main(int argc, char **argv)
 			break;
 		}
 	}
-	if (interactive || persist_cl) {
+	if (interactive)
 		ShowWindow(textwin.hWndParent, textwin.nCmdShow);
+	if (IsIconic(textwin.hWndParent)) { /* update icon */
+		RECT rect;
+		GetClientRect(textwin.hWndParent, (LPRECT) &rect);
+		InvalidateRect(textwin.hWndParent, (LPRECT) &rect, 1);
+		UpdateWindow(textwin.hWndParent);
 	}
-        if (IsIconic(textwin.hWndParent)) { /* update icon */
-                RECT rect;
-                GetClientRect(textwin.hWndParent, (LPRECT) &rect);
-                InvalidateRect(textwin.hWndParent, (LPRECT) &rect, 1);
-                UpdateWindow(textwin.hWndParent);
-        }
+# ifndef __WATCOMC__
+	/* Finally, also redirect C++ standard output streams. */
+	RedirectOutputStreams(TRUE);
+# endif
 #else /* WGP_CONSOLE */
 #ifdef CONSOLE_SWITCH_CP
         /* Change codepage of console to match that of the graph window.
@@ -600,7 +585,7 @@ int main(int argc, char **argv)
 #endif
 #endif
 
-	atexit(WinExit);
+	gp_atexit(WinExit);
 
 	if (!isatty(fileno(stdin)))
 		setmode(fileno(stdin), O_BINARY);
@@ -610,6 +595,7 @@ int main(int argc, char **argv)
 	/* First chance to close help system for console gnuplot,
 	   second for wgnuplot */
 	WinCloseHelp();
+	gp_exit_cleanup();
 	return 0;
 }
 
@@ -743,78 +729,72 @@ MyPutS(char *str)
 int
 MyFPrintF(FILE *file, const char *fmt, ...)
 {
-    int count;
-    va_list args;
+	int count;
+	va_list args;
 
-    va_start(args, fmt);
-    if (isterm(file)) {
-        char *buf;
-#ifdef __MSC__
-        count = _vscprintf(fmt, args) + 1;
-#else
-        count = vsnprintf(NULL,0,fmt,args) + 1;
-        if (count == 0) count = MAXPRINTF;
-#endif
-        va_end(args);
-        va_start(args, fmt);
-        buf = (char *)malloc(count * sizeof(char));
-        count = vsnprintf(buf, count, fmt, args);
-        TextPutS(&textwin, buf);
-        free(buf);
-    } else
-        count = vfprintf(file, fmt, args);
-    va_end(args);
-    return count;
+	va_start(args, fmt);
+	if (isterm(file)) {
+		char *buf;
+
+		count = vsnprintf(NULL, 0, fmt, args) + 1;
+		if (count == 0)
+			count = MAXPRINTF;
+		va_end(args);
+		va_start(args, fmt);
+		buf = (char *) malloc(count * sizeof(char));
+		count = vsnprintf(buf, count, fmt, args);
+		TextPutS(&textwin, buf);
+		free(buf);
+	} else {
+		count = vfprintf(file, fmt, args);
+	}
+	va_end(args);
+	return count;
 }
 
 int
 MyVFPrintF(FILE *file, const char *fmt, va_list args)
 {
-    int count;
+	int count;
 
-    if (isterm(file)) {
-        char *buf;
+	if (isterm(file)) {
+		char *buf;
+		va_list args_copied;
 
-#ifdef __MSC__
-        count = _vscprintf(fmt, args) + 1;
-#else
-        va_list args_copied;
-        va_copy(args_copied, args);
-        count = vsnprintf(NULL, 0U, fmt, args_copied) + 1;
-        if (count == 0) count = MAXPRINTF;
-        va_end(args_copied);
-#endif
-        buf = (char *)malloc(count * sizeof(char));
-        count = vsnprintf(buf, count, fmt, args);
-        TextPutS(&textwin, buf);
-        free(buf);
-    } else
-        count = vfprintf(file, fmt, args);
-    return count;
+		va_copy(args_copied, args);
+		count = vsnprintf(NULL, 0U, fmt, args) + 1;
+		if (count == 0)
+			count = MAXPRINTF;
+		va_end(args_copied);
+		buf = (char *) malloc(count * sizeof(char));
+		count = vsnprintf(buf, count, fmt, args);
+		TextPutS(&textwin, buf);
+		free(buf);
+	} else {
+		count = vfprintf(file, fmt, args);
+	}
+	return count;
 }
 
 int
 MyPrintF(const char *fmt, ...)
 {
-    int count;
-    char *buf;
-    va_list args;
+	int count;
+	char *buf;
+	va_list args;
 
-    va_start(args, fmt);
-#ifdef __MSC__
-    count = _vscprintf(fmt, args) + 1;
-#else
-    count = vsnprintf(NULL, 0, fmt, args) + 1;
-    if (count == 0) count = MAXPRINTF;
-#endif
-    va_end(args);
-    va_start(args, fmt);
-    buf = (char *)malloc(count * sizeof(char));
-    count = vsnprintf(buf, count, fmt, args);
-    TextPutS(&textwin, buf);
-    free(buf);
-    va_end(args);
-    return count;
+	va_start(args, fmt);
+	count = vsnprintf(NULL, 0, fmt, args) + 1;
+	if (count == 0)
+		count = MAXPRINTF;
+	va_end(args);
+	va_start(args, fmt);
+	buf = (char *) malloc(count * sizeof(char));
+	count = vsnprintf(buf, count, fmt, args);
+	TextPutS(&textwin, buf);
+	free(buf);
+	va_end(args);
+	return count;
 }
 
 size_t
@@ -844,6 +824,107 @@ MyFRead(void *ptr, size_t size, size_t n, FILE *file)
     return fread(ptr, size, n, file);
 }
 
+
+#ifdef USE_FAKEPIPES
+
+static char pipe_type = NUL;
+static char * pipe_filename = NULL;
+static char * pipe_command = NULL;
+
+FILE *
+fake_popen(const char * command, const char * type)
+{
+	FILE * f = NULL;
+	char tmppath[MAX_PATH];
+	char tmpfile[MAX_PATH];
+	DWORD ret;
+
+	if (type == NULL) return NULL;
+
+	pipe_type = NUL;
+	if (pipe_filename != NULL)
+		free(pipe_filename);
+
+	/* Random temp file name in %TEMP% */
+	ret = GetTempPath(sizeof(tmppath), tmppath);
+	if ((ret == 0) || (ret > sizeof(tmppath)))
+		return NULL;
+	ret = GetTempFileName(tmppath, "gpp", 0, tmpfile);
+	if (ret == 0)
+		return NULL;
+	pipe_filename = strdup(tmpfile);
+
+	if (*type == 'r') {
+		char * cmd;
+		int rc;
+		pipe_type = *type;
+		/* Execute command with redirection of stdout to temporary file. */
+		cmd = (char *) malloc(strlen(command) + strlen(pipe_filename) + 5);
+		sprintf(cmd, "%s > %s", command, pipe_filename);
+		rc = system(cmd);
+		free(cmd);
+		/* Now open temporary file. */
+		/* system() returns 1 if the command could not be executed. */
+		if (rc != 1)
+			f = fopen(pipe_filename, "r");
+		else {
+			remove(pipe_filename);
+			free(pipe_filename);
+			pipe_filename = NULL;
+			errno = EINVAL;
+		}
+	} else if (*type == 'w') {
+		pipe_type = *type;
+		/* Write output to temporary file and handle the rest in fake_pclose. */
+		if (type[1] == 'b')
+			int_error(NO_CARET, "Could not execute pipe '%s'. Writing to binary pipes is not supported.", command);
+		else
+			f = fopen(pipe_filename, "w");
+		pipe_command = strdup(command);
+	}
+
+	return f;
+}
+
+
+int fake_pclose(FILE *stream)
+{
+	int rc = 0;
+	if (!stream) return ECHILD;
+
+	/* Close temporary file */
+	fclose(stream);
+
+	/* Finally, execute command with redirected stdin. */
+	if (pipe_type == 'w') {
+		char * cmd;
+		cmd = (char *) gp_alloc(strlen(pipe_command) + strlen(pipe_filename) + 10, "fake_pclose");
+		/* FIXME: this won't work for binary data. We need a proper `cat` replacement. */
+		sprintf(cmd, "type %s | %s", pipe_filename, pipe_command);
+		rc = system(cmd);
+		free(cmd);
+	}
+
+	/* Delete temp file again. */
+	if (pipe_filename) {
+		remove(pipe_filename);
+		errno = 0;
+		free(pipe_filename);
+		pipe_filename = NULL;
+	}
+
+	if (pipe_command) {
+		/* system() returns 255 if the command could not be executed.
+		   The real popen would have returned an error already. */
+		if (rc == 255)
+			int_error(NO_CARET, "Could not execute pipe '%s'.", pipe_command);
+		free(pipe_command);
+	}
+
+	return rc;
+}
+#endif
+
 #else /* WGP_CONSOLE */
 
 
@@ -867,74 +948,81 @@ DWORD WINAPI stdin_pipe_reader(LPVOID param)
 
 int ConsoleGetch()
 {
-    int fd = fileno(stdin);
-    HANDLE h;
-    DWORD waitResult;
+	int fd = fileno(stdin);
+	HANDLE h;
+	DWORD waitResult;
 
-    if (!isatty(fd))
-        h = CreateThread(NULL, 0, stdin_pipe_reader, NULL, 0, NULL);
-    else
-        h = (HANDLE)_get_osfhandle(fd);
+	if (!isatty(fd))
+		h = CreateThread(NULL, 0, stdin_pipe_reader, NULL, 0, NULL);
+	else
+		h = (HANDLE)_get_osfhandle(fd);
 
-    do
-    {
-        waitResult = MsgWaitForMultipleObjects(1, &h, FALSE, INFINITE, QS_ALLINPUT);
-        if (waitResult == WAIT_OBJECT_0)
-        {
-            if (isatty(fd))
-            {
-                INPUT_RECORD rec;
-                DWORD recRead;
+	do {
+		waitResult = MsgWaitForMultipleObjects(1, &h, FALSE, INFINITE, QS_ALLINPUT);
+		if (waitResult == WAIT_OBJECT_0) {
+				DWORD c;
+			if (isatty(fd)) {
+				c = ConsoleReadCh();
+				if (c != NUL)
+					return c;
+			} else {
+				GetExitCodeThread(h, &c);
+				CloseHandle(h);
+				return c;
+			}
+		} else if (waitResult == WAIT_OBJECT_0+1) {
+			WinMessageLoop();
+			if (ctrlc_flag)
+				return '\r';
+		} else
+			break;
+	} while (1);
 
-                ReadConsoleInput(h, &rec, 1, &recRead);
-				/* FIXME: We should handle rec.Event.KeyEvent.wRepeatCount > 1, too. */
-                if (recRead == 1 && rec.EventType == KEY_EVENT && rec.Event.KeyEvent.bKeyDown &&
-                        (rec.Event.KeyEvent.wVirtualKeyCode < VK_SHIFT ||
-                         rec.Event.KeyEvent.wVirtualKeyCode > VK_MENU))
-                {
-                    if (rec.Event.KeyEvent.uChar.AsciiChar)
-						if ((rec.Event.KeyEvent.dwControlKeyState == SHIFT_PRESSED) && (rec.Event.KeyEvent.wVirtualKeyCode == VK_TAB))
-							return 034; /* remap Shift-Tab */
-						else
-							return rec.Event.KeyEvent.uChar.AsciiChar;
-                    else
-                        switch (rec.Event.KeyEvent.wVirtualKeyCode)
-                        {
-                            case VK_UP: return 020;
-                            case VK_DOWN: return 016;
-                            case VK_LEFT: return 002;
-                            case VK_RIGHT: return 006;
-                            case VK_HOME: return 001;
-                            case VK_END: return 005;
-                            case VK_DELETE: return 0117;
-                        }
-                }
-            }
-            else
-            {
-                DWORD c;
-                GetExitCodeThread(h, &c);
-                CloseHandle(h);
-                return c;
-            }
-        }
-        else if (waitResult == WAIT_OBJECT_0+1)
-        {
-            MSG msg;
-
-            while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
-            {
-                TranslateMessage(&msg);
-                DispatchMessage(&msg);
-            }
-        }
-        else
-            break;
-    } while (1);
+	return '\r';
 }
 
 #endif /* WGP_CONSOLE */
 
+
+int ConsoleReadCh()
+{
+	INPUT_RECORD rec;
+	DWORD recRead;
+	HANDLE h;
+
+	h = GetStdHandle(STD_INPUT_HANDLE);
+	if (h == NULL)
+		return NUL;
+
+	ReadConsoleInput(h, &rec, 1, &recRead);
+	/* FIXME: We should handle rec.Event.KeyEvent.wRepeatCount > 1, too. */
+	if (recRead == 1 && rec.EventType == KEY_EVENT && rec.Event.KeyEvent.bKeyDown &&
+			(rec.Event.KeyEvent.wVirtualKeyCode < VK_SHIFT ||
+				rec.Event.KeyEvent.wVirtualKeyCode > VK_MENU)) {
+		if (rec.Event.KeyEvent.uChar.AsciiChar) {
+			if ((rec.Event.KeyEvent.dwControlKeyState == SHIFT_PRESSED) && (rec.Event.KeyEvent.wVirtualKeyCode == VK_TAB))
+				return 034; /* remap Shift-Tab */
+			else
+				return rec.Event.KeyEvent.uChar.AsciiChar;
+		} else {
+			switch (rec.Event.KeyEvent.wVirtualKeyCode) {
+				case VK_UP: return 020;
+				case VK_DOWN: return 016;
+				case VK_LEFT: return 002;
+				case VK_RIGHT: return 006;
+				case VK_HOME: return 001;
+				case VK_END: return 005;
+				case VK_DELETE: return 0117;
+			}
+		}
+	}
+
+	/* Error reading event or, key up or, one of the following event records:
+	   MOUSE_EVENT_RECORD, WINDOW_BUFFER_SIZE_RECORD, MENU_EVENT_RECORD, FOCUS_EVENT_RECORD */
+	return NUL;
+}
+
+
 /* public interface to printer routines : Windows PRN emulation
  * (formerly in win.trm)
  */
@@ -945,22 +1033,22 @@ static char win_prntmp[MAX_PRT_LEN+1];
 FILE *
 open_printer()
 {
-    char *temp;
+	char *temp;
 
-    if ((temp = getenv("TEMP")) == (char *)NULL)
-        *win_prntmp='\0';
-    else  {
-        strncpy(win_prntmp,temp,MAX_PRT_LEN);
-        /* stop X's in path being converted by mktemp */
-        for (temp=win_prntmp; *temp; temp++)
-            *temp = tolower(*temp);
-        if ( strlen(win_prntmp) && (win_prntmp[strlen(win_prntmp)-1]!='\\') )
-            strcat(win_prntmp,"\\");
-    }
-    strncat(win_prntmp, "_gptmp",MAX_PRT_LEN-strlen(win_prntmp));
-    strncat(win_prntmp, "XXXXXX",MAX_PRT_LEN-strlen(win_prntmp));
-    mktemp(win_prntmp);
-    return fopen(win_prntmp, "w");
+	if ((temp = getenv("TEMP")) == (char *)NULL)
+		*win_prntmp = '\0';
+	else  {
+		safe_strncpy(win_prntmp, temp, MAX_PRT_LEN);
+		/* stop X's in path being converted by mktemp */
+		for (temp = win_prntmp; *temp != NUL; temp++)
+			*temp = tolower((unsigned char)*temp);
+		if ((strlen(win_prntmp) > 0) && (win_prntmp[strlen(win_prntmp) - 1] != '\\'))
+			strcat(win_prntmp, "\\");
+	}
+	strncat(win_prntmp, "_gptmp", MAX_PRT_LEN - strlen(win_prntmp));
+	strncat(win_prntmp, "XXXXXX", MAX_PRT_LEN - strlen(win_prntmp));
+	mktemp(win_prntmp);
+	return fopen(win_prntmp, "w");
 }
 
 void
@@ -1037,13 +1125,18 @@ WinWindowOpened(void)
 }
 
 
-/* returns true if there are any graph windows open (wxt/win terminals) */
+/* returns true if there are any graph windows open (wxt/caca/win terminals) */
+/* Note: This routine is used to handle "persist". Do not test for qt windows here 
+         since they run in a separate process */
 TBOOLEAN
 WinAnyWindowOpen(void)
 {
 	TBOOLEAN window_opened = WinWindowOpened();
 #ifdef WXWIDGETS
 	window_opened |= wxt_window_opened();
+#endif
+#ifdef HAVE_LIBCACA
+	window_opened |= CACA_window_opened();
 #endif
 	return window_opened;
 }
